(function(angular, $, _) {
  // Declare a list of dependencies.
  angular.module('form_processor', [
    'crmUi', 'crmUtil', 'ngRoute'
  ]);
  angular.module('form_processor').factory('formProcessorFactory', ["crmApi", "$q", function(crmApi, $q) {
    var actionTypes = {};

		var retrieveAction = function(name, context) {
			if (!(context in actionTypes)) {
				actionTypes[context] = {};
			}
			if (!(name in actionTypes[context])) {
				return crmApi('ActionProvider', 'getaction', {name: name, context:context});
			}
			return $q.resolve(actionTypes[context][name]);
		};
    
  	return {
			getAction: function(name, context) {
				return retrieveAction(name, context);
			},

  		getFieldList: function (inputs, actions, selectedAction, context) {
  		  var defer = $q.defer();
	  		var fields = [];
	  		var promises = [];
	  		angular.forEach(inputs, function(input, key) {
  				var field = {
  					'name': 'input.'+input.name,
  					'label': 'Input::'+input.name
  				};
  				fields.push(field);
				});
				
				var skipAfter = false;
				angular.forEach(actions, function(action, key) {
					if (selectedAction && selectedAction.name == action.name) {
						skipAfter = true;
					}
					if (!skipAfter && !action.delay) {
					  promises.push(
							retrieveAction(action.type, context)
					    .then(function (actionType) {
					      angular.forEach(actionType.output_spec, function(output_field, output_field_key) {
                  var field = {
                    'name': 'action.'+action.name+'.'+output_field.name,
                    'label': 'Action::'+action.title+'::'+output_field.title
                  };
                  fields.push(field);
                });
                return $q.resolve();
				      })
			      );
					}
				});
				
				$q.all(promises).then(function() {
				  defer.resolve(fields);
				});
				
				return defer.promise;
  		}
		};
  }]);
})(angular, CRM.$, CRM._);
