(function(angular, $, _) {

  function getValidators($scope) {
    var validators = []
    for(var validator_name in CRM.form_processor.validators) {
      var validator = CRM.form_processor.validators[validator_name];
      var exists = false;
      for (var i = 0; i < $scope.input.validators.length; i++) {
        if ($scope.input.validators[i].validator.name == validator.name) {
          exists = true;
          break;
        }
      }
      if (!exists) {
        validators.push(validator);
      }
    }
    return validators;
  }

  angular.module('form_processor').controller('InputDialogCtrl', function InputDialogCtrl($scope, dialogService, crmApi) {
    $scope.ts = CRM.ts(null);
    $scope.input = angular.copy($scope.model.input);
    $scope.locks = {name: true};
    $scope.isNameValid = false;
    $scope.validators = getValidators($scope);
    $scope.newValidator = null;
  	if (!$scope.input.deletedValidators) {
  		$scope.input.deletedValidators = [];
  	}

    $scope.$watch('input.name', function(newName, oldName) {
      // Watch for changes in the name field
      $scope.isNameValid = false;
      if (newName) {
        crmApi($scope.model.entity, 'validatename', {'name': newName, 'form_processor_instance_id': $scope.input.form_processor_instance_id, 'id': $scope.input.id})
          .then(function(data) {
            if (data.is_valid) {
              $scope.isNameValid = true;
            }
          });
      }
    }, true);

    $scope.$watch('input.title', function(newTitle, oldTitle) {
      // Watch for changes in the name field
      if ($scope.locks.name && $scope.input.id <= 0) {
        $scope.input.name = newTitle;
        $scope.input.name = $scope.input.name.toLowerCase();
        $scope.input.name = $scope.input.name.replace(/[^a-zA-Z\d]/g, '_');
        $scope.input.name = $scope.input.name.replace(/_+/g, '_');
      }
    }, true);

    $scope.isValidName = function(name) {
      for (var i = 0; i < $scope.model.inputs.length; i++) {
        if ($scope.model.inputs[i] != $scope.model.input) {
          if ($scope.model.inputs[i].name == name) {
            return false;
          }
        }
      }
      return (!name) || name.match(/^[a-z0-9_]+$/) ? true : false;
    };

    $scope.saveClick = function() {
      for(var i=0; i<$scope.input.validators.length; i++) {
        $scope.input.validators[i].configuration = {};
        for (var key in $scope.input.validators[i].validator.configuration) {
          $scope.input.validators[i].configuration[key] = $scope.input.validators[i].validator.configuration[key];
        }
      }
    	$scope.model.input = $scope.input;
    	dialogService.close('InputDialog', $scope.model);
    };

		$scope.cancelClick = function() {
    	dialogService.cancel('InputDialog');
    };

    $scope.addValidator = function(newValidator) {
    	if (!newValidator) {
    		return;
    	}
    	for (var i=0; i < $scope.input.validators.length; i++) {
    	    if ($scope.input.validators[i].validator.name == newValidator.name) {
    	      return;
          }
      }
      var validator = {
        validator: angular.copy(newValidator),
        configuration: angular.copy(newValidator.configuration_spec.default_configuration)
      };
      $scope.input.validators.push(validator);
      $scope.validators = getValidators($scope);
      return;
    };

    $scope.removeValidator = function(validator) {
    	var index = $scope.input.validators.indexOf(validator);
    	if (index >= 0) {
    		$scope.input.deletedValidators.push(validator);
    		$scope.input.validators.splice(index, 1);
        $scope.validators = getValidators($scope);
    	}
    };

  });

})(angular, CRM.$, CRM._);
